﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Resources;
using System.Threading;
using System.Reflection;
using System.IO;
using System.Globalization;
using RootPro.RootProCAD;
using RootPro.RootProCAD.Command;
using RootPro.RootProCAD.Geometry;
using RootPro.RootProCAD.UI;

///////////////////////////////////////////////////////////////////////////////////////////////////
// 選択した線の長さを測定するアドイン
// 選択した線分、連続線の線の長さを測定します。結果は一覧表で表示でき、線長の累計も計測できます。
///////////////////////////////////////////////////////////////////////////////////////////////////

namespace RCAddInMeasureCS
{
    [System.AddIn.AddIn("AppAddIn", Version = "1.0", Publisher = "", Description = "")]
    public partial class AppAddIn
    {
        string LengthCommandName = "RootPro.AddIn.LengthMesureCommand";     //コマンド名 
        FormResult formResult;

        /// <summary>
        /// ウィンドウハンドルを公開するためのインターフェイスの実装クラス
        /// </summary>
        class RootProWindow : System.Windows.Forms.IWin32Window
        {
            AppAddIn app;

            public RootProWindow(AppAddIn app)
            {
                this.app = app;
            }

            /// <summary>
            /// ウィンドウハンドルを取得する
            /// </summary>
            public virtual System.IntPtr Handle
            {
                get { return app.WindowHandle; }
            }
        }

        /// <summary>
        /// スタートアップ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AppAddIn_Startup(object sender, EventArgs e)
        {
			AppDomain.CurrentDomain.AssemblyResolve += new ResolveEventHandler(ResolveAssembly);

			//メニューにコマンド追加する
			CommandManager.AddMacroCommand("RCAddInMeasureCS", Properties.Resources.CommandName, MacroCommand);

            //結合コマンドを追加する
            Command command = CommandManager.AddCommand(LengthCommandName, Properties.Resources.CommandName);

            //パラメータを追加する
            CommandParameterGroup commandParameterGroup = command.ParameterGroups.Add(ParameterItemType.Shape, 0, Properties.Resources.ShapeParameterName);
            ShapeParameterItem parameterItem = (ShapeParameterItem)commandParameterGroup.CurrentItem; // パラメータアイテム
            parameterItem.ShapeTypeFilter = ShapeTypeFilters.Line | ShapeTypeFilters.Polyline;      //線分と連続線のみ
            parameterItem.Description = Properties.Resources.ShapeParameterDescription;

            //開始イベントと終了イベントの追加
            command.Begin += new CommandBeginEventHandler(command_Begin);
            command.End += new EventHandler(command_End);
        }

        /// <summary>
        /// 終了時
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AppAddIn_Shutdown(object sender, EventArgs e)
        {
            //メニューにコマンド追加する
            CommandManager.RemoveCommand(LengthCommandName);
            CommandManager.RemoveMacroCommand(MacroCommand);
        }

        #region VSTA generated code
        private void InternalStartup()
        {
            this.Startup += new System.EventHandler(AppAddIn_Startup);
            this.Shutdown += new System.EventHandler(AppAddIn_Shutdown);
        }
        #endregion

        /// <summary>
        /// マクロコマンドの実行
        /// </summary>
        private void MacroCommand()
        {

            //コマンドの実行
            CommandManager.ExecuteCommand(LengthCommandName);
        }

        /// <summary>
        /// コマンドの開始時に呼ばれる
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void command_Begin(Object sender, CommandBeginEventArgs e)
        {
            Command command = (Command)sender;
            RootProWindow wnd = new RootProWindow(this);

            //フォームの作成
            FormResult form = new FormResult();

            //フォームを表示する(モードレス)
            form.Show(wnd);

            formResult = form;


            //パラメータ変更イベントの追加
            command.ParameterChanged += new CommandParameterChangedEventHandler(command_ParameterChanged);
        }

        /// <summary>
        /// コマンドの終了時に呼ばれる
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void command_End(Object sender, EventArgs e)
        {
            Command command = (Command)sender;

            //パラメータ変更イベントの削除
            command.ParameterChanged -= new CommandParameterChangedEventHandler(command_ParameterChanged);

            //フォームは終了する
            formResult.Close();

            formResult.Dispose();

            formResult = null;

        }

        /// <summary>
        /// パラメータ変更イベント処理 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void command_ParameterChanged(Object sender, CommandParameterChangedEventArgs e)
        {
            Command command = (Command)sender;
            int id = e.ParameterID;// 変更のあったパラメータID
            ParameterItem parameterItem = command.GetParameterItemByID(id); // パラメータアイテム


            //図形１の取得
            ShapeParameterItem parameterItem0 = (ShapeParameterItem)command.GetParameterItemByID(0);
            if (parameterItem0.IsEmpty)
                return;

            Shape shape0 = parameterItem0.Shape.Shape;
            double length = 0.0;
            if (shape0 is LineShape)
            {
                length = ((LineShape)shape0).Length;
            }
            else if (shape0 is PolylineShape)
            {
                length = ((PolylineShape)shape0).Length;
            }

            // 結果を追加する
            if (formResult != null)
                formResult.AddResult(string.Format(Properties.Resources.ShapeLength, shape0.Number), length);

            //パラメータを空にする
            parameterItem0.SetEmpty();
        }

		private Assembly ResolveAssembly(object sender, ResolveEventArgs args)
		{
			AssemblyName assemblyName = new AssemblyName(args.Name);

			foreach (Assembly assembly in AppDomain.CurrentDomain.GetAssemblies())
			{
				if (AssemblyName.ReferenceMatchesDefinition(assemblyName, assembly.GetName()))
					return assembly;
			}

			if (assemblyName.Name.EndsWith(".resources"))
			{
				if (assemblyName.CultureName.Equals(Thread.CurrentThread.CurrentUICulture.Name, StringComparison.OrdinalIgnoreCase))
				{
					string dllPath = Path.GetDirectoryName(this.AddInFullPath);
					dllPath = Path.Combine(dllPath, assemblyName.CultureName);
					dllPath = Path.Combine(dllPath, assemblyName.Name + ".dll");

					if (File.Exists(dllPath))
					{
						try
						{
							return Assembly.LoadFrom(dllPath);
						}
						catch
						{
						}
					}
				}
			}

			return null;
		}
	}
}
